@@ -33,29 +33,31 @@ NetworkManagerDevice::NetworkManagerDevice(const QString &id, QSharedPointer<Net
     });
 
     m_statistics = m_device->deviceStatistics();
-    m_statistics->setRefreshRateMs(2000);
+
+    // We want to display speed in bytes per second, so use a fixed one-second
+    // update interval here so we are independant of the actual update rate of
+    // the daemon.
+    m_statistics->setRefreshRateMs(1000);
 
     // Unfortunately, the statistics interface does not emit change signals if
     // no change happened. This makes the change signals rather useless for our
     // case because we also need to know when no change happened, so that we
     // can update rate sensors to show 0. So instead use a timer and query the
-    // statistics every 2 seconds, updating the sensors as needed.
+    // statistics every second, updating the sensors as needed.
     m_statisticsTimer = std::make_unique<QTimer>();
-    m_statisticsTimer->setInterval(2000);
+    m_statisticsTimer->setInterval(1000);
     connect(m_statisticsTimer.get(), &QTimer::timeout, this, [this]() {
         auto newDownload = m_statistics->rxBytes();
         auto previousDownload = m_totalDownloadSensor->value().toULongLong();
         if (previousDownload > 0) {
-            // Our update interval is 2s, so to get bytes/sec we need to divide
-            // by 2.
-            m_downloadSensor->setValue((newDownload - previousDownload) / 2.0);
+            m_downloadSensor->setValue(newDownload - previousDownload);
         }
         m_totalDownloadSensor->setValue(newDownload);
 
         auto newUpload = m_statistics->txBytes();
         auto previousUpload = m_totalUploadSensor->value().toULongLong();
         if (previousUpload > 0) {
-            m_uploadSensor->setValue((newUpload - previousUpload) / 2.0);
+            m_uploadSensor->setValue(newUpload - previousUpload);
         }
         m_totalUploadSensor->setValue(newUpload);
     });
@@ -74,11 +76,11 @@ NetworkManagerDevice::NetworkManagerDevice(const QString &id, QSharedPointer<Net
     }
 
     if (m_device->type() == NetworkManager::Device::Wifi) {
-        auto wifi = m_device->as<NetworkManager::WirelessDevice>();
-        connect(wifi, &NetworkManager::WirelessDevice::activeConnectionChanged, this, [this, wifi]() { updateWifi(wifi); } );
-        connect(wifi, &NetworkManager::WirelessDevice::networkAppeared, this, [this, wifi]() { updateWifi(wifi); });
-        connect(wifi, &NetworkManager::WirelessDevice::networkDisappeared, this, [this, wifi]() { updateWifi(wifi); });
-        updateWifi(wifi);
+        m_wifiDevice = m_device->as<NetworkManager::WirelessDevice>();
+        connect(m_wifiDevice, &NetworkManager::WirelessDevice::activeConnectionChanged, this, &NetworkManagerDevice::updateWifi);
+        connect(m_wifiDevice, &NetworkManager::WirelessDevice::networkAppeared, this, &NetworkManagerDevice::updateWifi);
+        connect(m_wifiDevice, &NetworkManager::WirelessDevice::networkDisappeared, this, &NetworkManagerDevice::updateWifi);
+        updateWifi();
     }
 
     update();
@@ -90,6 +92,29 @@ NetworkManagerDevice::~NetworkManagerDevice()
 
 void NetworkManagerDevice::update()
 {
+    if (!m_device->activeConnection()) {
+        if (m_connected) {
+            m_connected = false;
+            if (m_statisticsTimer->isActive()) {
+                m_restoreTimer = true;
+                m_statisticsTimer->stop();
+            } else {
+                m_restoreTimer = false;
+            }
+            Q_EMIT disconnected(this);
+        }
+        return;
+    }
+
+    if (m_device->activeConnection() && !m_connected) {
+        m_connected = true;
+        if (m_restoreTimer) {
+            m_statisticsTimer->start();
+        }
+
+        Q_EMIT connected(this);
+    }
+
     setName(m_device->activeConnection()->connection()->name());
     m_networkSensor->setValue(name());
 
@@ -106,10 +131,19 @@ void NetworkManagerDevice::update()
     }
 }
 
-void NetworkManagerDevice::updateWifi(NetworkManager::WirelessDevice* device)
+bool NetworkManagerDevice::isConnected() const
 {
-    auto activeConnectionName = device->activeConnection()->connection()->name();
-    const auto networks = device->networks();
+    return m_connected;
+}
+
+void NetworkManagerDevice::updateWifi()
+{
+    if (!m_device->activeConnection()) {
+        return;
+    }
+
+    auto activeConnectionName = m_wifiDevice->activeConnection()->connection()->name();
+    const auto networks = m_wifiDevice->networks();
     std::for_each(networks.begin(), networks.end(), [this, activeConnectionName](QSharedPointer<NetworkManager::WirelessNetwork> network) {
         if (network->ssid() == activeConnectionName) {
             connect(network.data(), &NetworkManager::WirelessNetwork::signalStrengthChanged, m_signalSensor, &SensorProperty::setValue, Qt::UniqueConnection);
@@ -174,25 +208,18 @@ void NetworkManagerBackend::onDeviceAdded(const QString& uni)
             return;
     }
 
-    connect(device.get(), &NetworkManager::Device::activeConnectionChanged, this, [this, device, uni]() {
-        if (device->activeConnection()) {
-            onDeviceAdded(uni);
-        } else {
-            onDeviceRemoved(uni);
-        }
-    }, Qt::UniqueConnection);
-
-    if (!device->activeConnection()) {
-        return;
-    }
-
     if (m_devices.contains(uni)) {
         return;
     }
 
     auto nmDevice = new NetworkManagerDevice(device->interfaceName(), device);
+    connect(nmDevice, &NetworkManagerDevice::connected, this, &NetworkManagerBackend::deviceAdded);
+    connect(nmDevice, &NetworkManagerDevice::disconnected, this, &NetworkManagerBackend::deviceRemoved);
     m_devices.insert(uni, nmDevice);
-    Q_EMIT deviceAdded(nmDevice);
+
+    if (nmDevice->isConnected()) {
+        Q_EMIT deviceAdded(nmDevice);
+    }
 }
 
 void NetworkManagerBackend::onDeviceRemoved(const QString& uni)
@@ -202,7 +229,11 @@ void NetworkManagerBackend::onDeviceRemoved(const QString& uni)
     }
 
     auto device = m_devices.take(uni);
-    Q_EMIT deviceRemoved(device);
+
+    if (device->isConnected()) {
+        Q_EMIT deviceRemoved(device);
+    }
+
     delete device;
 }
 
